@@ -103,20 +103,31 @@ void ThNetworkThread::SendLoginToServer_ts(
 
 	ting::uint i = 0;
 
+	ASSERT(i < p.Size())
 	p[i] = 0;//login message ID
 	++i;
 
+	ASSERT(i + 2 <= p.Size())
 	ting::ToNetworkFormat16(u16(username.size()), &p[i]);
-//	TRACE(<< "unLen = " << u32(p[i]) << " " << u32(p[i + 1]) << std::endl)
+	TRACE(<< "unLen = " << u32(p[i]) << " " << u32(p[i + 1]) << std::endl)
 	i += 2;
-	memcpy(&p[i], username.c_str(), username.size());
-	i += username.size();
 
+	if(username.size() > 0){
+		ASSERT(i < p.Size() && i + username.size() <= p.Size())
+		memcpy(&p[i], username.c_str(), username.size());
+		i += username.size();
+	}
+
+	ASSERT(i + 2 <= p.Size())
 	ting::ToNetworkFormat16(u16(password.size()), &p[i]);
-//	TRACE(<< "pwLen = " << u32(p[i]) << " " << u32(p[i + 1]) << std::endl)
+	TRACE(<< "pwLen = " << u32(p[i]) << " " << u32(p[i + 1]) << std::endl)
 	i += 2;
-	memcpy(&p[i], password.c_str(), password.size());
-	ASSERT(i + password.size() == p.SizeInBytes())
+
+	if(password.size() > 0){
+		ASSERT(i < p.Size() && i + password.size() <= p.Size())
+		memcpy(&p[i], password.c_str(), password.size());
+		ASSERT(i + password.size() == p.Size())
+	}
 
 	this->SendData_ts(p);
 }
@@ -147,7 +158,7 @@ void MainThread::SetUpReconnectTimerIfAutologinIsOn(){
 
 
 void ConnectedMessage::Handle(){
-	TRACE(<< "C_ConnectedMessage::Handle(): invoked" << std::endl)
+	TRACE(<< "ConnectedMessage::Handle(): invoked" << std::endl)
 
 	if(this->reason == cliser::NetworkThread::SUCCESS){
 		this->mt->ChangeNetworkState(MainThread::CONNECTED);
@@ -167,7 +178,7 @@ void ConnectedMessage::Handle(){
 
 
 void DisconnectedMessage::Handle(){
-	TRACE(<<"C_DisconnectedMessage::Handle(): invoked"<<std::endl)
+	TRACE(<<"DisconnectedMessage::Handle(): invoked"<<std::endl)
 
 	//if state is already OFFLINE_LOGIN_FAILED then do not change it to simple OFFLINE
 	if(this->mt->networkState != MainThread::OFFLINE_LOGIN_FAILED){
@@ -183,7 +194,7 @@ void DisconnectedMessage::Handle(){
 
 void NetworkDataFromServerMessage::Handle(){
 //	TRACE(<<"C_NetworkDataFromServerMessage::Handle(): invoked"<<std::endl)
-	ASSERT(this->data.SizeInBytes() > 0)
+	ASSERT(this->data.Size() > 0)
 	switch(this->data[0]){
 		case 0x00: //login result
 			this->mt->HandleLoginResultFromServer(this->data);
@@ -403,10 +414,19 @@ void ThNetworkThread::SendListenNextToServer_ts(){
 
 //override
 void ConnectRequestMessage::Handle(){
-//	TRACE(<< "ConnectRequestMessage::HANDLE(): invoked" << std::endl)
+	TRACE(<< "ConnectRequestMessage::HANDLE(): invoked" << std::endl)
 
-	if(this->initializeReconnectTime){
+	if(this->manuallyRequestedConnect){
 		this->t->reconnectTime = MainThread::DMinReconnectTime();
+
+		//If previous login attempt failed at login then
+		//change the state to simple OFFLINE, because
+		//OFFLINE_LOGIN_FAILED is to prevent further automatic logins,
+		//while we have manually requested login here, so we do not want
+		//the login to be stopped.
+		if(this->t->networkState == MainThread::OFFLINE_LOGIN_FAILED){
+			this->t->networkState = MainThread::OFFLINE;
+		}
 	}
 
 	//Check if already connecting or connected
@@ -427,6 +447,8 @@ void ConnectRequestMessage::Handle(){
 
 //override
 void LoginRequestMessage::Handle(){
+	TRACE(<< "LoginRequestMessage::Handle(): invoked" << std::endl)
+
 	//Check if already loggin in or logged in
 	if(this->t->networkState == MainThread::LOGGING_IN ||
 			this->t->networkState == MainThread::LOGGED_IN
@@ -434,14 +456,17 @@ void LoginRequestMessage::Handle(){
 	{
 		//Notify listeners by changing state to current state (no state change in fact).
 		this->t->ChangeNetworkState(this->t->networkState);
+		TRACE(<< "LoginRequestMessage::Handle(): already logged in, returning" << std::endl)
 		return;
 	}
 
 	if(this->t->networkState != MainThread::CONNECTED){
 		//ignore request, do not alter network state
+		TRACE(<< "LoginRequestMessage::Handle(): state is not CONNECTED, returning" << std::endl)
 		return;
 	}
 
+//	TRACE(<< "LoginRequestMessage::Handle(): sending login request" << std::endl)
 	
 	//Send login request to server and change current network state to LOGGING_IN.
 	this->t->networkThread.SendLoginToServer_ts(
@@ -449,15 +474,18 @@ void LoginRequestMessage::Handle(){
 			Preferences::Inst().GetPassword()
 		);
 
+//	TRACE(<< "LoginRequestMessage::Handle(): login request sent" << std::endl)
+
 	this->t->ChangeNetworkState(MainThread::LOGGING_IN);
+//	TRACE(<< "LoginRequestMessage::Handle(): exit" << std::endl)
 }
 
 
 
-void MainThread::ConnectInternal_ts(bool initReconnectTime){
+void MainThread::ConnectInternal_ts(bool manuallyRequested){
 	this->PushMessage(
 			Ptr<Message>(
-					new ConnectRequestMessage(this, initReconnectTime)
+					new ConnectRequestMessage(this, manuallyRequested)
 				)
 		);
 }
@@ -505,7 +533,7 @@ void MainThread::HandleLoginResultFromServer(ting::Array<ting::byte> p){
 
 	ASSERT(p[0] == 0)// 0 = login result
 
-	if(p.SizeInBytes() != 2){
+	if(p.Size() != 2){
 		ASSERT_INFO(false, "bad formed login result packet received")
 		return;//ignore
 	}
@@ -540,7 +568,7 @@ void MainThread::HandleListeningMusicDataFromServer(ting::Array<ting::byte> p){
 void MainThread::HandlePlayerSwitchMessageFromServer(ting::Array<ting::byte> p){
 	ting::uint i = 0;
 
-	if(p.Size() < i + 1){
+	if(i + 1 > p.Size()){
 		//bad formed packet, ignore
 		ASSERT_INFO(false, "bad formed packet")
 		return;
@@ -554,7 +582,7 @@ void MainThread::HandlePlayerSwitchMessageFromServer(ting::Array<ting::byte> p){
 	//parse player username
 	std::string playerUsername;
 	{
-		if(p.Size() < i + 2){
+		if(i + 2 > p.Size()){
 			//bad formed packet, ignore
 			ASSERT_INFO(false, "bad formed packet")
 			return;
@@ -562,7 +590,13 @@ void MainThread::HandlePlayerSwitchMessageFromServer(ting::Array<ting::byte> p){
 		ting::uint unLen = ting::FromNetworkFormat16(&p[i]);
 		i += 2;
 
-		if(p.Size() < i + unLen){
+		if(unLen == 0){
+			ASSERT_INFO(false, "empty username not allowed")
+			return;//ignore
+		}
+
+		ASSERT(unLen != 0)
+		if(i + unLen > p.Size()){
 			//bad formed packet, ignore
 			ASSERT_INFO(false, "bad formed packet")
 			return;
