@@ -121,7 +121,7 @@ private:
 	void ChangeNetworkState(ENetworkState newState);
 
 
-	void ConnectInternal_ts(bool initReconnectTime);
+	void ConnectInternal_ts(bool manuallyRequested);
 
 public:
 	inline void Connect_ts(){
@@ -343,12 +343,12 @@ public:
 class ConnectRequestMessage : public ting::Message{
 	MainThread *t;
 
-	bool initializeReconnectTime;
+	bool manuallyRequestedConnect;
 
 public:
-	ConnectRequestMessage(MainThread* mt, bool initReconnectTime = false) :
+	ConnectRequestMessage(MainThread* mt, bool manuallyRequested = false) :
 			t(mt),
-			initializeReconnectTime(initReconnectTime)
+			manuallyRequestedConnect(manuallyRequested)
 	{
 		ASSERT(this->t)
 	}
