@@ -30,17 +30,9 @@ PlaybackThread::PlaybackThread() :
 
 
 
-struct MusicAction{
-	ting::byte id;
-	ting::u32 timestamp;
-	ting::byte volume;
-	float freq;
-};
-
-
-
 //retuns true if data is not corrupted
-static bool ParseMusicData(Array<MusicAction>& ma, const byte* dataStart, const byte* const dataEnd){
+//static
+bool PlaybackThread::ParseMusicData(Array<MusicAction>& ma, const byte* dataStart, const byte* const dataEnd){
 	ASSERT(ma.Size() > 0)
 	ASSERT(dataStart < dataEnd)
 
@@ -56,7 +48,7 @@ static bool ParseMusicData(Array<MusicAction>& ma, const byte* dataStart, const
 		curMa->timestamp = ting::FromNetworkFormat32(dataStart);
 		dataStart += 4;
 
-		if(curMa->id == 0){ //note on action ID
+		if(curMa->id == NOTE_ON){ //note on action ID
 			if(dataEnd - dataStart < 1 + 4){
 				//error, data corrupted
 				break;
@@ -80,7 +72,7 @@ static bool ParseMusicData(Array<MusicAction>& ma, const byte* dataStart, const
 
 void PlaybackThread::Run(){
 	ting::byte curInstrumentCode;
-	Array<MusicAction> curMusicData;
+//	Array<MusicAction> curMusicData;
 	
 	MusicAction *curMusicAction;
 
@@ -93,7 +85,7 @@ void PlaybackThread::Run(){
 		}
 
 		//curMusicData.Size() = 0 indicates that it is time to start new music data chunk
-		if(curMusicData.Size() == 0){
+		if(this->curMusicData.Size() == 0){
 			//if queue is empty then hang on GetMsg() waiting for some event, then check again
 			if(this->musicDataQueue.size() == 0){
 				//turn off playing note if player has switched
@@ -133,16 +125,21 @@ void PlaybackThread::Run(){
 //			TRACE(<< "PlaybackThread::Run(): numActions = " << numActions << std::endl)
 
 			ASSERT(numActions > 0)
-			curMusicData.Init(numActions);
-			if(!ParseMusicData(curMusicData, &startOfMusicData[4], endOfMusicData)){
-				curMusicData.Reset();
+			this->curMusicData.Init(numActions);
+			if(!PlaybackThread::ParseMusicData(
+					this->curMusicData,
+					&startOfMusicData[4],
+					endOfMusicData
+				))
+			{
+				this->curMusicData.Reset();
 				this->musicDataQueue.pop_front();//pop music data
 				ASSERT(false)
 			}
 
-			ASSERT(curMusicData.Size() > 0)
+			ASSERT(this->curMusicData.Size() > 0)
 
-			curMusicAction = curMusicData.Begin();
+			curMusicAction = this->curMusicData.Begin();
 
 			//init timestamp bias taking timestamp of first action in chunk
 			this->musicDataQueue.pop_front();//pop music data
@@ -160,7 +157,7 @@ void PlaybackThread::Run(){
 		//check if music action triggers
 		u32 ticks = ting::GetTicks();
 		if(ticks - localTimeOfStreamStart >= curMusicAction->timestamp - timestampBias){
-			ASSERT(curMusicAction < curMusicData.End())
+			ASSERT(curMusicAction < this->curMusicData.End())
 			switch(curMusicAction->id){
 				case NOTE_ON: //note on
 					if(this->ch.IsNotValid()){
@@ -183,8 +180,8 @@ void PlaybackThread::Run(){
 
 			//move to next music action
 			++curMusicAction;
-			if(curMusicAction >= curMusicData.End()){
-				curMusicData.Reset();
+			if(curMusicAction >= this->curMusicData.End()){
+				this->curMusicData.Reset();
 			}
 		}else{
 			//did not trigger, wait
@@ -231,3 +228,21 @@ void PushMusicDataMessage::Handle(){
 	this->t->musicDataQueue.push_back(this->data);
 }
 
+
+
+void PlaybackThread::ResetPlayback_ts(){
+	this->PushMessage(Ptr<Message>(
+			new ResetPlaybackMessage(this)
+		));
+}
+
+
+
+//override
+void ResetPlaybackMessage::Handle(){
+	this->t->curMusicData.Reset();
+	this->t->musicDataQueue.clear();
+
+	if(this->t->ch.IsValid())
+		this->t->ch->Stop();
+}
