@@ -28,9 +28,11 @@ THE SOFTWARE. */
  * Created on February 21, 2009, 1:50 PM
  */
 
-#ifndef M_Synth_hpp
-#define	M_Synth_hpp
+#pragma once
 
+
+#include <ting/Array.hpp>
+#include <ting/types.hpp>
 #include <ting/Fixed.hpp>
 
 #include "../aumiks.hpp"
@@ -39,8 +41,11 @@ THE SOFTWARE. */
 
 namespace aumiks{
 
+
 using namespace ting;
 
+
+
 class Instrument : public aumiks::Sound{
 public:
 
@@ -172,6 +177,8 @@ public:
 
 
 class SampledInstrument : public Instrument{
+	friend class Channel;
+
 public:
 
 #ifdef M_USE_FIXED_POINT
@@ -180,7 +187,21 @@ public:
 	typedef float real;
 #endif
 
+
+protected:
+
+	struct ADSRParams{
+		real attack;//periods
+		real decay;//periods
+		real sustainLevel;
+		real release;//periods
+	};
+
+public:
+
 	class Channel : public Instrument::Channel{
+		Ref<SampledInstrument> instr;
+
 		real curPhase;
 //
 		real curTime; //in milliseconds
@@ -190,17 +211,19 @@ public:
 			SUSTAIN,
 			RELEASE
 		} envPhase;
-//
-//		real phaseStep;// (Hz / 44100) * 2pi
-//		real attackTime;//ms
-//		real decayTime;//ms
-//		real sustainLevel;
-//		real releaseTime;//ms
-//
-//		real releaseFromLevel;
+
+		//position in sample buffer of the instrument
+		real phaseStep;// (Hz / 44100) * 2pi
+
+		real attackTime;
+		real decayTime;
+		real releaseTime;
+
+		real releaseFromLevel;
 	public:
 
-		Channel() :
+		Channel(Ref<SampledInstrument> instrument) :
+				instr(instrument),
 				curPhase(0),
 				curTime(0),
 				envPhase(ATTACK)
@@ -211,12 +234,15 @@ public:
 		//override
 		// freq in Hz
 		void SetFrequency(float freq){
-//			this->phaseStep = D2Pi<real>() * real(freq) / real(44100);
-//			this->attackTime = 1000.0f * 20.0f / freq;
-//			this->decayTime = 1000.0f * 20.0f / freq;
-//			this->sustainLevel = 0.4f;
-//			this->releaseTime = 1000.0f * 30.0f / freq;
-		};
+			real frequency(freq);
+			ADSRParams &p = this->instr->adsrParams;
+
+			this->phaseStep = frequency / this->instr->sampleFreq;
+			
+			this->attackTime = real(1000.0f) * p.attack / frequency;
+			this->decayTime = real(1000.0f) * p.decay / frequency;
+			this->releaseTime = real(1000.0f) * p.release / frequency;
+		}
 
 	private:
 		//override
@@ -226,12 +252,40 @@ public:
 	//override
 	Ref<Instrument::Channel> CreateChannel();
 
-//	static inline ting::Ref<SampledInstrument> New(){
-//		return ting::Ref<SampledInstrument>(new SampledInstrument());
-//	}
+protected:
+	SampledInstrument(
+			Array<s16> sampleData,
+			real sampleFrequency,
+			ADSRParams adsrParameters
+		) :
+			sample(sampleData),
+			sampleFreq(sampleFrequency),
+			adsrParams(adsrParameters)
+	{}
+
+private:
+	Array<real> sample;//sampling rate 44100Hz
+
+	real sampleFreq;
+
+	ADSRParams adsrParams;
 };
 
 
 
+//class TromboneInstrument : public SampledInstrument{
+//	TromboneInstrument() :
+//			SampledInstrument()
+//	{
+//		//TODO:
+//	}
+//
+//public:
+//	static inline ting::Ref<TromboneInstrument> New(){
+//		return ting::Ref<TromboneInstrument>(new TromboneInstrument());
+//	}
+//};
+
+
+
 }//~namespace
-#endif	//~once
